@@ -1,12 +1,25 @@
 //-----------------------------------------------------------------------------
-// Copyright (c) 2016, 2020, Oracle and/or its affiliates. All rights reserved.
-// This program is free software: you can modify it and/or redistribute it
-// under the terms of:
+// Copyright (c) 2016, 2022, Oracle and/or its affiliates.
 //
-// (i)  the Universal Permissive License v 1.0 or at your option, any
-//      later version (http://oss.oracle.com/licenses/upl); and/or
+// This software is dual-licensed to you under the Universal Permissive License
+// (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl and Apache License
+// 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose
+// either license.
 //
-// (ii) the Apache License v 2.0. (http://www.apache.org/licenses/LICENSE-2.0)
+// If you elect to accept the software under the Apache License, Version 2.0,
+// the following applies:
+//
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//    https://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
 //-----------------------------------------------------------------------------
 
 //-----------------------------------------------------------------------------
@@ -56,7 +69,7 @@ extern "C" {
 
 // define ODPI-C version information
 #define DPI_MAJOR_VERSION   4
-#define DPI_MINOR_VERSION   3
+#define DPI_MINOR_VERSION   6
 #define DPI_PATCH_LEVEL     0
 #define DPI_VERSION_SUFFIX
 
@@ -277,7 +290,8 @@ typedef uint32_t dpiOracleTypeNum;
 #define DPI_ORACLE_TYPE_JSON_OBJECT                 2028
 #define DPI_ORACLE_TYPE_JSON_ARRAY                  2029
 #define DPI_ORACLE_TYPE_UROWID                      2030
-#define DPI_ORACLE_TYPE_MAX                         2031
+#define DPI_ORACLE_TYPE_LONG_NVARCHAR               2031
+#define DPI_ORACLE_TYPE_MAX                         2032
 
 // session pool close modes
 typedef uint32_t dpiPoolCloseMode;
@@ -412,6 +426,7 @@ typedef struct dpiVar dpiVar;
 //-----------------------------------------------------------------------------
 // Forward Declarations of Other Types
 //-----------------------------------------------------------------------------
+typedef struct dpiAccessToken dpiAccessToken;
 typedef struct dpiAppContext dpiAppContext;
 typedef struct dpiCommonCreateParams dpiCommonCreateParams;
 typedef struct dpiConnCreateParams dpiConnCreateParams;
@@ -422,6 +437,7 @@ typedef struct dpiDataTypeInfo dpiDataTypeInfo;
 typedef struct dpiEncodingInfo dpiEncodingInfo;
 typedef struct dpiErrorInfo dpiErrorInfo;
 typedef struct dpiJsonNode dpiJsonNode;
+typedef struct dpiMsgRecipient dpiMsgRecipient;
 typedef struct dpiObjectAttrInfo dpiObjectAttrInfo;
 typedef struct dpiObjectTypeInfo dpiObjectTypeInfo;
 typedef struct dpiPoolCreateParams dpiPoolCreateParams;
@@ -439,6 +455,13 @@ typedef struct dpiVersionInfo dpiVersionInfo;
 typedef struct dpiXid dpiXid;
 
 
+//-----------------------------------------------------------------------------
+// Declaration for function pointers
+//-----------------------------------------------------------------------------
+typedef int (*dpiAccessTokenCallback)(void *context,
+        dpiAccessToken *accessToken);
+
+
 //-----------------------------------------------------------------------------
 // Complex Native Data Types (used for transferring data to/from ODPI-C)
 //-----------------------------------------------------------------------------
@@ -553,6 +576,7 @@ struct dpiCommonCreateParams {
     uint32_t driverNameLength;
     int sodaMetadataCache;
     uint32_t stmtCacheSize;
+    dpiAccessToken *accessToken;
 };
 
 // structure used for creating connections
@@ -610,6 +634,14 @@ struct dpiDataTypeInfo {
     dpiObjectType *objectType;
 };
 
+// structure used for storing token authentication data
+struct dpiAccessToken {
+    const char *token;
+    uint32_t tokenLength;
+    const char *privateKey;
+    uint32_t privateKeyLength;
+};
+
 // structure used for transferring encoding information from ODPI-C
 struct dpiEncodingInfo {
     const char *encoding;
@@ -649,6 +681,8 @@ struct dpiObjectTypeInfo {
     int isCollection;
     dpiDataTypeInfo elementTypeInfo;
     uint16_t numAttributes;
+    const char *packageName;
+    uint32_t packageNameLength;
 };
 
 // structure used for creating pools
@@ -669,6 +703,8 @@ struct dpiPoolCreateParams {
     const char *plsqlFixupCallback;
     uint32_t plsqlFixupCallbackLength;
     uint32_t maxSessionsPerShard;
+    dpiAccessTokenCallback accessTokenCallback;
+    void *accessTokenCallbackContext;
 };
 
 // structure used for transferring query metadata from ODPI-C
@@ -679,6 +715,12 @@ struct dpiQueryInfo {
     int nullOk;
 };
 
+// structure used for recipients list
+struct dpiMsgRecipient {
+    const char *name;
+    uint32_t nameLength;
+};
+
 // structure used for sharding key columns
 struct dpiShardingKeyColumn {
     dpiOracleTypeNum oracleTypeNum;
@@ -764,6 +806,8 @@ struct dpiSubscrMessage {
     uint32_t queueNameLength;
     const char *consumerName;
     uint32_t consumerNameLength;
+    const void *aqMsgId;
+    uint32_t aqMsgIdLength;
 };
 
 // structure used for transferring query information in messages in
@@ -963,6 +1007,13 @@ DPI_EXPORT int dpiConn_newDeqOptions(dpiConn *conn, dpiDeqOptions **options);
 // create a new enqueue options object and return it
 DPI_EXPORT int dpiConn_newEnqOptions(dpiConn *conn, dpiEnqOptions **options);
 
+// create a new, empty JSON
+DPI_EXPORT int dpiConn_newJson(dpiConn *conn, dpiJson **json);
+
+// create a new AQ queue with JSON payload
+DPI_EXPORT int dpiConn_newJsonQueue(dpiConn *conn, const char *name,
+        uint32_t nameLength, dpiQueue **queue);
+
 // create a new message properties object and return it
 DPI_EXPORT int dpiConn_newMsgProps(dpiConn *conn, dpiMsgProps **props);
 
@@ -1428,6 +1479,9 @@ DPI_EXPORT int dpiMsgProps_getOriginalMsgId(dpiMsgProps *props,
 DPI_EXPORT int dpiMsgProps_getPayload(dpiMsgProps *props, dpiObject **obj,
         const char **value, uint32_t *valueLength);
 
+// return the payload of the message (JSON)
+DPI_EXPORT int dpiMsgProps_getPayloadJson(dpiMsgProps *props, dpiJson **json);
+
 // return the priority of the message
 DPI_EXPORT int dpiMsgProps_getPriority(dpiMsgProps *props, int32_t *value);
 
@@ -1460,6 +1514,9 @@ DPI_EXPORT int dpiMsgProps_setOriginalMsgId(dpiMsgProps *props,
 DPI_EXPORT int dpiMsgProps_setPayloadBytes(dpiMsgProps *props,
         const char *value, uint32_t valueLength);
 
+// set the payload of the message (as JSON)
+DPI_EXPORT int dpiMsgProps_setPayloadJson(dpiMsgProps *props, dpiJson *json);
+
 // set the payload of the message (as an object)
 DPI_EXPORT int dpiMsgProps_setPayloadObject(dpiMsgProps *props,
         dpiObject *obj);
@@ -1467,6 +1524,9 @@ DPI_EXPORT int dpiMsgProps_setPayloadObject(dpiMsgProps *props,
 // set the priority of the message
 DPI_EXPORT int dpiMsgProps_setPriority(dpiMsgProps *props, int32_t value);
 
+// set recipients associated with the message
+DPI_EXPORT int dpiMsgProps_setRecipients(dpiMsgProps *props,
+        dpiMsgRecipient *recipients, uint32_t numRecipients);
 
 //-----------------------------------------------------------------------------
 // Object Methods (dpiObject)
@@ -1627,6 +1687,9 @@ DPI_EXPORT int dpiPool_getPingInterval(dpiPool *pool, int *value);
 // release a reference to the pool
 DPI_EXPORT int dpiPool_release(dpiPool *pool);
 
+// set token parameter for token based authentication
+DPI_EXPORT int dpiPool_setAccessToken(dpiPool *pool, dpiAccessToken *params);
+
 // set the pool's "get" mode
 DPI_EXPORT int dpiPool_setGetMode(dpiPool *pool, dpiPoolGetMode value);
 
