@@ -1,12 +1,25 @@
 /*-----------------------------------------------------------------------------
- * Copyright (c) 2016, 2018, Oracle and/or its affiliates. All rights reserved.
- * This program is free software: you can modify it and/or redistribute it
- * under the terms of:
+ * Copyright (c) 2016, 2022, Oracle and/or its affiliates.
  *
- * (i)  the Universal Permissive License v 1.0 or at your option, any
- *      later version (http://oss.oracle.com/licenses/upl); and/or
+ * This software is dual-licensed to you under the Universal Permissive License
+ * (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl and Apache License
+ * 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose
+ * either license.*
  *
- * (ii) the Apache License v 2.0. (http://www.apache.org/licenses/LICENSE-2.0)
+ * If you elect to accept the software under the Apache License, Version 2.0,
+ * the following applies:
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *    https://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  *---------------------------------------------------------------------------*/
 
 /*-----------------------------------------------------------------------------
@@ -32,11 +45,11 @@ CREATE DIRECTORY &dir_name AS '&dir_path';
 alter session set nls_date_format = 'YYYY-MM-DD HH24:MI:SS';
 alter session set nls_numeric_characters='.,';
 
-create user &main_user identified by &main_password
+create user &main_user identified by "&main_password"
 quota unlimited on users
 default tablespace users;
 
-create user &proxy_user identified by &proxy_password;
+create user &proxy_user identified by "&proxy_password";
 alter user &proxy_user grant connect through &main_user;
 
 grant create session to &proxy_user;
